-- Copyright (c) 2012 Cloudera, Inc. All rights reserved.
--
-- Version 35 of the CMF schema. Migration code to be applied on top of the
-- previous version.
--

update SCHEMA_VERSION set VERSION=35;

--
-- Remove the non-null constraint for the service id associated with a config
-- revision since a revision can be tied to either a service or a host.
-- Create a new column in CONFIG_REVISIONS to store host id
--
alter table CONFIG_REVISIONS
      alter column SERVICE_ID drop not null;

alter table CONFIG_REVISIONS
      add column HOST_ID int8;

--
--
-- Remove the non-null constraint for the service id associated with a config
-- since a config can be tied to either a service or a host.
-- Create a new column in CONFIGS to store host id
--
alter table CONFIGS
      alter column SERVICE_ID drop not null;

alter table CONFIGS
      add column HOST_ID int8;

--
-- Each host with configs must be tied to a configuration revision. When the
-- configs are modified, a new revision is created and all non-inherited
-- configs are added to it.
-- Create a new column in HOSTS to refer to CONFIG_REVISIONS
--
alter table HOSTS
      add column CONFIG_REVISION_ID int8;

-- As preparation to  insert a base revision for existing hosts,
-- create a sequence that can be used for auto incrementing the REVISION_ID
-- field in the CONFIG_REVISIONS table.
-- The sequence is initialized to the maximum value of REVISION_ID from the
-- CONFIG_REVISIONS table.
-- After inserting base revisions for hosts, the sequence is dropped.
--
create sequence CONFIG_REVISIONS_REVISION_ID_SEQ;

select setval('CONFIG_REVISIONS_REVISION_ID_SEQ', max(REVISION_ID)) from CONFIG_REVISIONS;

alter table CONFIG_REVISIONS
      alter column REVISION_ID set default nextval('CONFIG_REVISIONS_REVISION_ID_SEQ');

--
-- Insert a base revision for each existing host.
-- After inserting revisions for each host, the sequence is dropped.
--
insert into CONFIG_REVISIONS (HOST_ID, CREATED_INSTANT, MESSAGE)
       select HOST_ID, extract(epoch FROM now()), 'Base revision'
       from HOSTS;

alter table CONFIG_REVISIONS
       alter column REVISION_ID drop default;

drop sequence CONFIG_REVISIONS_REVISION_ID_SEQ;

--
-- Link each host to its current config revision.
--
-- Note that UPDATE doesn't allow aliases in older versions of Postgres, so
-- they aren't used here.
update HOSTS
       set CONFIG_REVISION_ID=
       (select REVISION_ID
       from CONFIG_REVISIONS r
       where HOSTS.HOST_ID=r.HOST_ID);

--
-- All data insertion is done. Set up indices and constraints.
--
create index IDX_HOST_CONFIG_REVISION on HOSTS (CONFIG_REVISION_ID);

create index IDX_CONFIG_REVISION_HOST on CONFIG_REVISIONS (HOST_ID);

create index IDX_CONFIG_HOST on CONFIGS (HOST_ID);

alter table HOSTS
      add constraint FK_HOST_CONFIG_REVISION
      foreign key (CONFIG_REVISION_ID)
      references CONFIG_REVISIONS (REVISION_ID);

alter table CONFIG_REVISIONS
      add constraint FK_CONFIG_REVISION_HOST
      foreign key (HOST_ID)
      references HOSTS (HOST_ID);

alter table CONFIGS
      add constraint FK_CONFIG_HOST
      foreign key (HOST_ID)
      references HOSTS (HOST_ID);
