-- Copyright (c) 2012 Cloudera, Inc. All rights reserved.
--
-- Version 4509 of the CMF schema.
--
-- Create base role config groups as needed and update existing roles
-- to use them.
--

update SCHEMA_VERSION set VERSION=4509;

-- Need a sequence so we can auto increment group IDs.
create sequence ROLE_CONFIG_GROUP_ID_SEQ;

-- We set the initial value to be the max of the ROLE_CONFIG_GROUP_ID column
-- so that the first row we insert will have the very next ID.
select setval('ROLE_CONFIG_GROUP_ID_SEQ', max(ROLE_CONFIG_GROUP_ID))
  from ROLE_CONFIG_GROUPS;

-- Now we modify the column to use the sequence for default values.
alter table ROLE_CONFIG_GROUPS
  alter column ROLE_CONFIG_GROUP_ID
  set default nextval('ROLE_CONFIG_GROUP_ID_SEQ');

-- Insert new groups for each service/roletype combination (from existing
-- roles) that's not already represented by a group.
insert into ROLE_CONFIG_GROUPS (SERVICE_ID, ROLE_TYPE, NAME)
  (select distinct r.SERVICE_ID, r.ROLE_TYPE, r.ROLE_TYPE || '_BASE'
  from ROLES r left join ROLE_CONFIG_GROUPS g
  on r.SERVICE_ID=g.SERVICE_ID and r.ROLE_TYPE=g.ROLE_TYPE
  where g.SERVICE_ID is null);

-- Done inserting, remove the default value and drop the sequence.
alter table ROLE_CONFIG_GROUPS
  alter column ROLE_CONFIG_GROUP_ID drop default;
drop sequence ROLE_CONFIG_GROUP_ID_SEQ;

-- Update all roles to point to the appropriate role config group.
-- Note that UPDATE doesn't allow aliases in older versions of Postgres, so
-- they aren't used here.
update ROLES
  set ROLE_CONFIG_GROUP_ID=(
    select ROLE_CONFIG_GROUP_ID
    from ROLE_CONFIG_GROUPS g
    where ROLES.ROLE_TYPE=g.ROLE_TYPE and
          ROLES.SERVICE_ID=g.SERVICE_ID);

-- Now that every role has a group, make the foreign key not nullable.
alter table ROLES
  alter column ROLE_CONFIG_GROUP_ID set not null;
